/*
 * nbexit.c  -  Handle program exit and cleanup
 *
 * Copyright (C) 2003-2006 Gero Kuhlmann <gero@gkminix.han.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Id: nbexit.c,v 1.4 2005/11/13 20:14:34 gkminix Exp $
 */

#define __IN_NBEXIT 1
#include <common.h>
#include <nblib.h>
#include "privlib.h"



/* Type of function to call on exit */
typedef void (*exithandler) __P((void));



/* Maximum number of exit functions allowed */
#define MAX_ATEXIT	32


/* Local variables */
static int isinit = FALSE;			/* initialization flag */
static int handlernum = 0;			/* current no. of functions */
static exithandler handlers[MAX_ATEXIT];	/* atexit handler list */



/*
 * Execute all exit routines, but do not actually exit the program
 */
void nblib_exit_cleanup()
{
  while (handlernum > 0)
	(handlers[--handlernum])();
}



/*
 * Function to be called upon program termination
 */
#if defined(HAVE_ATEXIT) || defined(HAVE_ON_EXIT)
# if !defined(HAVE_ATEXIT)
static void termall(exitcode, exitarg)
int exitcode;
voidstar exitarg;
{
  nblib_exit_cleanup();
}
# else
#  define termall	nblib_exit_cleanup
# endif
#endif



/*
 * Exit the current program. This routine is only needed in case
 * neither atexit() not on_exit() is available.
 */
#if !defined(HAVE_ATEXIT) && !defined(HAVE_ON_EXIT)
void nbexit(code)
int code;
{
  nblib_exit_cleanup();
  exit(code);
}
#endif



/*
 * Add a routine to the list of functions to execute at exit
 */
int nbatexit(func)
void (*func)();
{
  /* Check if termination handler already initialized */
  if (!isinit) {
#ifdef HAVE_ATEXIT
	if (atexit(&termall) != 0)
		return(-1);
#else
# ifdef HAVE_ON_EXIT
	if (on_exit(&termall, NULL) != 0)
		return(-1);
# endif
#endif
	isinit = TRUE;
  }

  /* Save function to execute at exit */
  if (handlernum >= MAX_ATEXIT)
	return(-1);
  handlers[handlernum++] = (exithandler)func;
  return(0);
}

